@@ -266,7 +266,7 @@ QDateTime DateTimeGrid::startDateTime() const
 void DateTimeGrid::setStartDateTime( const QDateTime& dt )
 {
     d->startDateTime = dt;
-    emit gridChanged();
+    Q_EMIT gridChanged();
 }
 
 
@@ -292,14 +292,14 @@ void DateTimeGrid::setDayWidth( qreal w )
 {
     assert( w>0 );
     d->dayWidth = w;
-    emit gridChanged();
+    Q_EMIT gridChanged();
 }
 
 
 void DateTimeGrid::setScale( Scale s )
 {
     d->scale = s;
-    emit gridChanged();
+    Q_EMIT gridChanged();
 }
 
 
@@ -313,7 +313,7 @@ void DateTimeGrid::setUserDefinedLowerScale( DateTimeScaleFormatter* lower )
 {
     delete d->lower;
     d->lower = lower;
-    emit gridChanged();
+    Q_EMIT gridChanged();
 }
 
 
@@ -321,7 +321,7 @@ void DateTimeGrid::setUserDefinedUpperScale( DateTimeScaleFormatter* upper )
 {
     delete d->upper;
     d->upper = upper;
-    emit gridChanged();
+    Q_EMIT gridChanged();
 }
 
 
@@ -340,7 +340,7 @@ DateTimeScaleFormatter* DateTimeGrid::userDefinedUpperScale() const
 void DateTimeGrid::setWeekStart( Qt::DayOfWeek ws )
 {
     d->weekStart = ws;
-    emit gridChanged();
+    Q_EMIT gridChanged();
 }
 
 
@@ -353,7 +353,7 @@ Qt::DayOfWeek DateTimeGrid::weekStart() const
 void DateTimeGrid::setFreeDays( const QSet<Qt::DayOfWeek>& fd )
 {
     d->freeDays = fd;
-    emit gridChanged();
+    Q_EMIT gridChanged();
 }
 
 
@@ -389,7 +389,7 @@ void DateTimeGrid::setRowSeparators( bool enable )
 void DateTimeGrid::setNoInformationBrush( const QBrush& brush )
 {
     d->noInformationBrush = brush;
-    emit gridChanged();
+    Q_EMIT gridChanged();
 }
 
 
@@ -472,7 +472,7 @@ bool DateTimeGrid::mapFromChart( const Span& span, const QModelIndex& idx,
     QDateTime st = d->chartXtoDateTime(span.start());
     QDateTime et = d->chartXtoDateTime(span.start()+span.length());
     //qDebug() << "DateTimeGrid::mapFromChart("<<span<<") => "<< st << et;
-    Q_FOREACH( const Constraint& c, constraints ) {
+    for ( const Constraint& c : constraints ) {
         if ( c.type() != Constraint::TypeHard || !isSatisfiedConstraint( c )) continue;
         if ( c.startIndex() == idx ) {
             QDateTime tmpst = model()->data( c.endIndex(), StartTimeRole ).toDateTime();
@@ -597,35 +597,60 @@ void DateTimeGrid::Private::paintVerticalLines( QPainter* painter,
 void DateTimeGrid::Private::paintVerticalUserDefinedLines( QPainter* painter,
                                                            const QRectF& sceneRect,
                                                            const QRectF& exposedRect,
-                                                           const DateTimeScaleFormatter* formatter,
                                                            QWidget* widget )
 {
     Q_UNUSED( widget );
-    QDateTime dt = chartXtoDateTime( exposedRect.left() );
-    dt = formatter->currentRangeBegin( dt );
+    DateTimeScaleFormatter *lowerFormatter, *upperFormatter;
+    switch ( scale ) {
+        case ScaleUserDefined:
+            lowerFormatter = lower;
+            upperFormatter = upper;
+            break;
+        default:
+            getAutomaticFormatters( &lowerFormatter, &upperFormatter );
+            break;
+    }
     QPen pen = painter->pen();
     pen.setBrush( QApplication::palette().dark() );
-    pen.setStyle( Qt::DashLine );
-    painter->setPen( pen );
-    for ( qreal x = dateTimeToChartX( dt ); x < exposedRect.right();
-        dt = formatter->nextRangeBegin( dt ),x=dateTimeToChartX( dt ) ) {
-        if ( freeDays.contains( static_cast<Qt::DayOfWeek>( dt.date().dayOfWeek() ) ) ) {
-            QBrush oldBrush = painter->brush();
-            if (freeDaysBrush.style() == Qt::NoBrush)
-                painter->setBrush( widget?widget->palette().midlight()
-                                 :QApplication::palette().midlight() );
-            else
-                painter->setBrush(freeDaysBrush);
 
-          painter->fillRect( QRectF( x, exposedRect.top(), dayWidth, exposedRect.height() ), painter->brush() );
-          painter->setBrush( oldBrush );
+    // Do freeDays, we need to iterate over all dates
+    QDateTime dtLeft = chartXtoDateTime( exposedRect.left() );
+    if ( ! freeDays.isEmpty() ) {
+        QDate lastDate = chartXtoDateTime( exposedRect.right() ).date();
+        for (QDateTime date(dtLeft.date(), QTime()); date.date() <= lastDate; date = date.addDays(1)) {
+            if ( freeDays.contains( static_cast<Qt::DayOfWeek>( date.date().dayOfWeek() ) ) ) {
+                if (freeDaysBrush.style() == Qt::NoBrush) {
+                    painter->setBrush( widget?widget->palette().midlight():QApplication::palette().midlight() );
+                } else {
+                    painter->setBrush(freeDaysBrush);
+                }
+                qreal x = dateTimeToChartX( date );
+                painter->fillRect( QRectF( x, exposedRect.top(), dayWidth, exposedRect.height() ), painter->brush() );
+            }
         }
-              //TODO not the best solution as it might be one paint too much, but i don't know what
-              //causes the test to fail yet, i think it might be a rounding error
-        //if ( x >= exposedRect.left() ) {
-            // FIXME: Also fill area between this and the next vertical line to indicate free days? (Johannes)
-    painter->drawLine( QPointF( x, sceneRect.top() ), QPointF( x, sceneRect.bottom() ) );
-        //}
+    }
+    QDateTime dt = upperFormatter->currentRangeBegin( dtLeft );
+    // Get all upper scale gridline x values to avoid mixing with lower scale gridlines
+    QList<qreal> upperXList;
+    for ( qreal x = dateTimeToChartX( dt ); x < exposedRect.right(); dt = upperFormatter->nextRangeBegin( dt ), x=dateTimeToChartX( dt ) ) {
+        upperXList.append(x);
+    }
+    dt = lowerFormatter->currentRangeBegin( dtLeft );
+    for ( qreal x = dateTimeToChartX( dt ); x < exposedRect.right();
+        dt = lowerFormatter->nextRangeBegin( dt ),x=dateTimeToChartX( dt ) )
+    {
+        if (!upperXList.contains(x)) {
+            pen.setStyle( Qt::DashLine );
+            painter->setPen( pen );
+            painter->drawLine( QPointF( x, sceneRect.top() ), QPointF( x, sceneRect.bottom() ) );
+        }
+    }
+    dt = upperFormatter->currentRangeBegin( dtLeft );
+    for ( qreal x : upperXList )
+    {
+        pen.setStyle( Qt::SolidLine );
+        painter->setPen( pen );
+        painter->drawLine( QPointF( x, sceneRect.top() ), QPointF( x, sceneRect.bottom() ) );
     }
 }
 
@@ -654,7 +679,6 @@ void DateTimeGrid::paintGrid( QPainter* painter,
                               AbstractRowController* rowController,
                               QWidget* widget )
 {
-    // TODO: Support hours and weeks
     switch ( scale() ) {
     case ScaleHour:
     case ScaleDay:
@@ -662,27 +686,9 @@ void DateTimeGrid::paintGrid( QPainter* painter,
     case ScaleMonth:
         d->paintVerticalLines( painter, sceneRect, exposedRect, widget, d->headerTypeForScale( scale() ) );
         break;
-    case ScaleAuto: {
-        const qreal tabw = QApplication::fontMetrics().boundingRect( QLatin1String( "XXXXX" ) ).width();
-        const qreal dayw = dayWidth();
-        if ( dayw > 24*60*60*tabw ) {
-
-            d->paintVerticalUserDefinedLines( painter, sceneRect, exposedRect, &d->minute_lower, widget );
-        } else if ( dayw > 24*60*tabw ) {
-            d->paintVerticalLines( painter, sceneRect, exposedRect, widget, Private::HeaderHour );
-        } else if ( dayw > 24*tabw ) {
-        d->paintVerticalLines( painter, sceneRect, exposedRect, widget, Private::HeaderDay );
-        } else if ( dayw > tabw ) {
-            d->paintVerticalUserDefinedLines( painter, sceneRect, exposedRect, &d->week_lower, widget );
-        } else if ( 4*dayw > tabw ) {
-            d->paintVerticalUserDefinedLines( painter, sceneRect, exposedRect, &d->month_lower, widget );
-        } else {
-            d->paintVerticalUserDefinedLines( painter, sceneRect, exposedRect, &d->year_lower, widget );
-        }
-        break;
-    }
+    case ScaleAuto:
     case ScaleUserDefined:
-        d->paintVerticalUserDefinedLines( painter, sceneRect, exposedRect, d->lower, widget );
+        d->paintVerticalUserDefinedLines( painter, sceneRect, exposedRect, widget );
         break;
     }
     if ( rowController ) {
@@ -715,6 +721,7 @@ int DateTimeGrid::Private::tabHeight( const QString& txt, QWidget* widget ) cons
 {
     QStyleOptionHeader opt;
     if ( widget ) opt.initFrom( widget );
+    else opt.palette = QApplication::palette();
     opt.text = txt;
     QStyle* style;
     if ( widget ) style = widget->style();
@@ -748,6 +755,57 @@ void DateTimeGrid::Private::getAutomaticFormatters( DateTimeScaleFormatter** low
     }
 }
 
+void DateTimeGrid::Private::getFormatters( DateTimeScaleFormatter** lower, DateTimeScaleFormatter** upper)
+{
+    switch ( scale ) {
+        case DateTimeGrid::ScaleHour:
+            *lower = &hour_lower;
+            *upper = &hour_upper;
+            break;
+        case DateTimeGrid::ScaleDay:
+            *lower = &day_lower;
+            *upper = &day_upper;
+            break;
+        case DateTimeGrid::ScaleWeek:
+            *lower = &week_lower;
+            *upper = &week_upper;
+            break;
+        case DateTimeGrid::ScaleMonth:
+            *lower = &month_lower;
+            *upper = &month_upper;
+            break;
+        case DateTimeGrid::ScaleUserDefined:
+            *lower = this->lower;
+            *upper = this->upper;
+            break;
+        default: /*ScaleAuto:*/
+            getAutomaticFormatters( lower, upper );
+            break;
+    }
+}
+
+DateTimeGrid::HeaderType DateTimeGrid::sectionHandleAtPos(int x, int y, const QRect &headerRect) const
+{
+    QDateTime dt1 = d->chartXtoDateTime( x );
+    QDateTime dt2 = d->chartXtoDateTime( x + 5 );
+
+    DateTimeScaleFormatter *lower, *upper;
+    const_cast<Private*>(d)->getFormatters( &lower, &upper );
+
+    const qreal lowerHeight = d->tabHeight( lower->text( dt1 ) );
+    const qreal upperHeight = d->tabHeight( upper->text( dt1 ) );
+    const qreal upperRatio = upperHeight/( lowerHeight+upperHeight );
+
+    const QRectF upperHeaderRect( x, headerRect.top(), 5, headerRect.height() * upperRatio );
+    const QRectF lowerHeaderRect( x, upperHeaderRect.bottom()+1, 5,  headerRect.height()-upperHeaderRect.height()-1 );
+    if ( upperHeaderRect.contains( QPoint( x , y ) ) ) {
+        return upper->currentRangeBegin(dt2) == upper->nextRangeBegin(dt1) ? UpperHeader : NoHeader;
+    }
+    if (lowerHeaderRect.contains( QPoint( x , y ) ) ) {
+        return lower->currentRangeBegin(dt2)==lower->nextRangeBegin(dt1) ? LowerHeader : NoHeader;
+    }
+    return NoHeader;
+}
 
 void DateTimeGrid::paintHeader( QPainter* painter,  const QRectF& headerRect, const QRectF& exposedRect,
                                 qreal offset, QWidget* widget )
@@ -818,6 +876,7 @@ void DateTimeGrid::paintUserDefinedHeader( QPainter* painter,
 
         QStyleOptionHeader opt;
         if ( widget ) opt.init( widget );
+        else opt.palette = QApplication::palette();
         opt.rect = QRectF( x - offset+1, headerRect.top(), qMax<qreal>( 1., nextx-x-1 ), headerRect.height() ).toAlignedRect();
         opt.textAlignment = formatter->alignment();
         opt.text = formatter->text( dt );
@@ -874,6 +933,7 @@ void DateTimeGrid::Private::paintHeader( QPainter* painter,
           x = dateTimeToChartX( dt ) ) {
         QStyleOptionHeader opt;
         if ( widget ) opt.init( widget );
+        else opt.palette = QApplication::palette();
         opt.rect = formatter->textRect( x, offset, dayWidth, headerRect, dt );
         opt.text = formatter->format( dt );
         opt.textAlignment = Qt::AlignCenter;
