@@ -33,6 +33,7 @@
 #include <QPrinter>
 #include <QItemSelectionModel>
 #include <QGraphicsSceneMouseEvent>
+#include <QGuiApplication>
 
 #include <cassert>
 
@@ -47,9 +48,10 @@
 using namespace KGantt;
 
 HeaderWidget::HeaderWidget( GraphicsView* parent )
-    : QWidget( parent ), m_offset( 0. )
+    : QWidget( parent ), m_offset( 0. ), m_headerType( DateTimeGrid::NoHeader )
 {
     assert( parent ); // Parent must be set
+    setMouseTracking(true);
 }
 
 HeaderWidget::~HeaderWidget()
@@ -83,6 +85,96 @@ bool HeaderWidget::event( QEvent* event )
     return QWidget::event( event );
 }
 
+void HeaderWidget::mousePressEvent(QMouseEvent *event)
+{
+    DateTimeGrid* const grid = qobject_cast< DateTimeGrid* >( view()->grid() );
+    int mousePosX = event->pos().x() + view()->horizontalScrollBar()->value();
+    m_headerType = grid->sectionHandleAtPos( mousePosX, event->pos().y(), geometry() );
+    if (m_headerType != DateTimeGrid::NoHeader) {
+        bool hasCursor = testAttribute(Qt::WA_SetCursor);
+        if (!hasCursor) {
+            setCursor(QCursor(Qt::SplitHCursor));
+        }
+        m_mousePosX = mousePosX;
+        event->accept();
+        return;
+    }
+    QWidget::mousePressEvent( event );
+}
+
+void HeaderWidget::mouseReleaseEvent(QMouseEvent *event)
+{
+    if ( m_headerType > 0 ) {
+        DateTimeGrid* const grid = qobject_cast< DateTimeGrid* >( view()->grid() );
+        int mousePosX = event->pos().x() + view()->horizontalScrollBar()->value();
+        if ( grid->sectionHandleAtPos( mousePosX, event->pos().y(), geometry() ) == DateTimeGrid::NoHeader ) {
+            bool hasCursor = testAttribute(Qt::WA_SetCursor);
+            if (hasCursor) {
+                unsetCursor();
+            }
+        }
+        m_headerType = DateTimeGrid::NoHeader;
+        m_mousePosX = mousePosX;
+        QGuiApplication::restoreOverrideCursor();
+    }
+    QWidget::mouseReleaseEvent(event);
+}
+
+void HeaderWidget::mouseMoveEvent(QMouseEvent *event)
+{
+    DateTimeGrid* const grid = qobject_cast< DateTimeGrid* >( view()->grid() );
+    int mousePosX = event->pos().x() + view()->horizontalScrollBar()->value();
+    switch ( m_headerType ) {
+        case DateTimeGrid::UpperHeader:
+        {
+            if ( mousePosX > m_mousePosX )  {
+                grid->setDayWidth( qMax<qreal>( 1.0, grid->dayWidth() * 1.05 ) );
+            } else {
+                grid->setDayWidth( qMax<qreal>( 1.0, grid->dayWidth() / 1.05 ) );
+            }
+            m_mousePosX = mousePosX;
+            event->accept();
+            return;
+        }
+        case DateTimeGrid::LowerHeader:
+            if ( mousePosX > m_mousePosX )  {
+                grid->setDayWidth( qMax<qreal>( 1.0, grid->dayWidth() * 1.01 ) );
+            } else {
+                grid->setDayWidth( qMax<qreal>( 1.0, grid->dayWidth() / 1.01 ) );
+            }
+            m_mousePosX = mousePosX;
+            event->accept();
+            return;
+        default: {
+            bool hasCursor = testAttribute(Qt::WA_SetCursor);
+            DateTimeGrid::HeaderType type = grid->sectionHandleAtPos( mousePosX, event->pos().y(), geometry());
+            if (type != DateTimeGrid::NoHeader) {
+                if (!hasCursor) {
+                    setCursor(QCursor(Qt::SplitHCursor));
+                }
+                event->accept();
+                return;
+            }
+            if (hasCursor) {
+                unsetCursor();
+            }
+            break;
+        }
+    }
+    QWidget::mouseMoveEvent(event);
+}
+
+void HeaderWidget::wheelEvent( QWheelEvent *event )
+{
+    DateTimeGrid* const grid = qobject_cast< DateTimeGrid* >( view()->grid() );
+    if ( event->angleDelta().y() > 0 ) {
+        grid->setDayWidth( qMax<qreal>( 1.0, grid->dayWidth() * 1.1 ) );
+    } else {
+        grid->setDayWidth( qMax<qreal>( 1.0, grid->dayWidth() / 1.1 ) );
+    }
+    event->accept();
+}
+
 void HeaderWidget::contextMenuEvent( QContextMenuEvent* event )
 {
     QMenu contextMenu;
@@ -205,6 +297,10 @@ GraphicsView::Private::Private( GraphicsView* _q )
 {
 }
 
+GraphicsView::Private::~Private()
+{
+}
+
 void GraphicsView::Private::updateHeaderGeometry()
 {
     q->setViewportMargins(0,rowcontroller->headerHeight(),0,0);
@@ -289,8 +385,8 @@ void GraphicsView::Private::removeConstraintsRecursive( QAbstractProxyModel *sum
     }
     //qDebug() << "removing constraints from"<<index;
     // NOTE: Constraints are mapped to indexes in the summaryModel->sourceModel()
-    QList<Constraint> clst = scene.constraintModel()->constraintsForIndex( summaryModel->mapToSource( index ) );
-    Q_FOREACH( Constraint c, clst ) {
+    const QList<Constraint> clst = scene.constraintModel()->constraintsForIndex( summaryModel->mapToSource( index ) );
+    for ( const Constraint &c : clst ) {
         scene.constraintModel()->removeConstraint( c );
     }
 }
@@ -322,28 +418,27 @@ void GraphicsView::Private::slotRowsRemoved( const QModelIndex& parent,  int sta
 void GraphicsView::Private::slotItemClicked( const QModelIndex& idx )
 {
     QModelIndex sidx = idx;//scene.summaryHandlingModel()->mapToSource( idx );
-    emit q->clicked( sidx );
+    Q_EMIT q->clicked( sidx );
     if (q->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, nullptr, q))
-        emit q->activated( sidx );
+        Q_EMIT q->activated( sidx );
 }
 
 void GraphicsView::Private::slotItemDoubleClicked( const QModelIndex& idx )
 {
     QModelIndex sidx = idx;//scene.summaryHandlingModel()->mapToSource( idx );
-    emit q->qrealClicked( sidx );
+    Q_EMIT q->qrealClicked( sidx );
     if (!q->style()->styleHint(QStyle::SH_ItemView_ActivateItemOnSingleClick, nullptr, q))
-        emit q->activated( sidx );
+        Q_EMIT q->activated( sidx );
 }
 
 void GraphicsView::Private::slotHeaderContextMenuRequested( const QPoint& pt )
 {
-    emit q->headerContextMenuRequested( headerwidget.mapToGlobal( pt ) );
+    Q_EMIT q->headerContextMenuRequested( headerwidget.mapToGlobal( pt ) );
 }
 
 GraphicsView::GraphicsView( QWidget* parent )
     : QGraphicsView( parent ), _d( new Private( this ) )
 {
-
 #if defined KDAB_EVAL
   EvalDialog::checkEvalLicense( "KD Gantt" );
 #endif
@@ -516,11 +611,7 @@ AbstractGrid* GraphicsView::grid() const
 
 AbstractGrid* GraphicsView::takeGrid()
 {
-    AbstractGrid *grid = d->scene.grid();
-    if ( grid ) {
-        d->scene.setGrid( nullptr) ;
-    }
-    return grid;
+    return d->scene.takeGrid();
 }
 
 
@@ -694,5 +785,9 @@ void GraphicsView::print( QPainter* painter, qreal start, qreal end,
   d->scene.print(painter, start, end, targetRect, drawRowLabels, drawColumnLabels);
 }
 
+void GraphicsView::printDiagram( QPrinter *printer, const PrintingContext &context )
+{
+    d->scene.printDiagram( printer, context );
+}
 
 #include "moc_kganttgraphicsview.cpp"
